@@ -1,5 +1,5 @@
 // FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
-//go:build go1.23
+//go:build go1.21
 
 package loader
 
@@ -18,15 +18,14 @@ import (
 	"github.com/docker/cli/cli/compose/template"
 	"github.com/docker/cli/cli/compose/types"
 	"github.com/docker/cli/opts"
-	"github.com/docker/cli/opts/swarmopts"
 	"github.com/docker/docker/api/types/versions"
 	"github.com/docker/go-connections/nat"
-	"github.com/docker/go-units"
+	units "github.com/docker/go-units"
 	"github.com/go-viper/mapstructure/v2"
 	"github.com/google/shlex"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
-	"gopkg.in/yaml.v3"
+	yaml "gopkg.in/yaml.v2"
 )
 
 // Options supported by Load
@@ -54,11 +53,11 @@ func ParseYAML(source []byte) (map[string]any, error) {
 	if err := yaml.Unmarshal(source, &cfg); err != nil {
 		return nil, err
 	}
-	_, ok := cfg.(map[string]any)
+	cfgMap, ok := cfg.(map[any]any)
 	if !ok {
 		return nil, errors.Errorf("top-level object must be a mapping")
 	}
-	converted, err := convertToStringKeysRecursive(cfg, "")
+	converted, err := convertToStringKeysRecursive(cfgMap, "")
 	if err != nil {
 		return nil, err
 	}
@@ -270,7 +269,7 @@ type ForbiddenPropertiesError struct {
 	Properties map[string]string
 }
 
-func (*ForbiddenPropertiesError) Error() string {
+func (e *ForbiddenPropertiesError) Error() string {
 	return "Configuration contains forbidden properties"
 }
 
@@ -350,20 +349,24 @@ func createTransformHook(additionalTransformers ...Transformer) mapstructure.Dec
 
 // keys needs to be converted to strings for jsonschema
 func convertToStringKeysRecursive(value any, keyPrefix string) (any, error) {
-	if mapping, ok := value.(map[string]any); ok {
+	if mapping, ok := value.(map[any]any); ok {
 		dict := make(map[string]any)
 		for key, entry := range mapping {
+			str, ok := key.(string)
+			if !ok {
+				return nil, formatInvalidKeyError(keyPrefix, key)
+			}
 			var newKeyPrefix string
 			if keyPrefix == "" {
-				newKeyPrefix = key
+				newKeyPrefix = str
 			} else {
-				newKeyPrefix = fmt.Sprintf("%s.%s", keyPrefix, key)
+				newKeyPrefix = fmt.Sprintf("%s.%s", keyPrefix, str)
 			}
 			convertedEntry, err := convertToStringKeysRecursive(entry, newKeyPrefix)
 			if err != nil {
 				return nil, err
 			}
-			dict[key] = convertedEntry
+			dict[str] = convertedEntry
 		}
 		return dict, nil
 	}
@@ -382,6 +385,16 @@ func convertToStringKeysRecursive(value any, keyPrefix string) (any, error) {
 	return value, nil
 }
 
+func formatInvalidKeyError(keyPrefix string, key any) error {
+	var location string
+	if keyPrefix == "" {
+		location = "at top level"
+	} else {
+		location = "in " + keyPrefix
+	}
+	return errors.Errorf("non-string key %s: %#v", location, key)
+}
+
 // LoadServices produces a ServiceConfig map from a compose file Dict
 // the servicesDict is not validated if directly used. Use Load() to enable validation
 func LoadServices(servicesDict map[string]any, workingDir string, lookupEnv template.Mapping) ([]types.ServiceConfig, error) {
@@ -926,7 +939,7 @@ func toServicePortConfigs(value string) ([]any, error) {
 
 	for _, key := range keys {
 		// Reuse ConvertPortToPortConfig so that it is consistent
-		portConfig, err := swarmopts.ConvertPortToPortConfig(nat.Port(key), portBindings)
+		portConfig, err := opts.ConvertPortToPortConfig(nat.Port(key), portBindings)
 		if err != nil {
 			return nil, err
 		}
